import React, { useRef, useEffect, useCallback, useState } from 'react';
import classnames from 'classnames';
import { useScrollObserver } from '@highlight-ui/utils-hooks';
import { parseMetadata, suffixMetadata } from '@highlight-ui/utils-commons';
import { IconButton, Button } from '@highlight-ui/button';
import { Tooltip } from '@highlight-ui/tooltip';
import { Typography } from '@highlight-ui/typography';
import { GroupedPortal } from '@highlight-ui/utils-portal-manager';
import noop from 'lodash/noop';
import invoke from 'lodash/invoke';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var styles$1 = {"dialog":"Dialog-module__2icvnh15__v6-1-0","size-small":"Dialog-module__3spVEVua__v6-1-0","size-medium":"Dialog-module__-zWxw4HX__v6-1-0","size-large":"Dialog-module__3wZ2NPV0__v6-1-0","header":"Dialog-module__2wAUH40a__v6-1-0","headerShadowed":"Dialog-module__xODFOi1M__v6-1-0","headerTitle":"Dialog-module__1lxoaZF6__v6-1-0","closeButton":"Dialog-module__2l9SSdo___v6-1-0","body":"Dialog-module__1bPY4avA__v6-1-0","footer":"Dialog-module__3o8NTS6m__v6-1-0","footerShadowed":"Dialog-module__1s03rY8s__v6-1-0","disabled":"Dialog-module__1TM_UCZw__v6-1-0"};

function useEventListener(t,r,i,o){void 0===i&&(i=global),void 0===o&&(o={});var c=useRef(),u=o.capture,a=o.passive,v=o.once;useEffect(function(){c.current=r;},[r]),useEffect(function(){if(i&&i.addEventListener){var e=function(e){return c.current(e)},n={capture:u,passive:a,once:v};return i.addEventListener(t,e,n),function(){i.removeEventListener(t,e,n);}}},[t,i,u,a,v]);}

const TABBABLE_NODES = /input|select|textarea|button|object/;
const FOCUS_SELECTOR = "a, input, select, textarea, button, object, [tabindex]";
function hidden(element) {
  if (process.env.NODE_ENV === "test") {
    return false;
  }
  return element.style.display === "none";
}
function visible(element) {
  let parentElement = element;
  while (parentElement) {
    if (parentElement === document.body) {
      break;
    }
    if (hidden(parentElement)) {
      return false;
    }
    parentElement = parentElement.parentNode;
  }
  return true;
}
function getElementTabIndex(element) {
  let tabIndex = element.getAttribute("tabindex");
  if (tabIndex === null) {
    tabIndex = void 0;
  }
  return parseInt(tabIndex, 10);
}
function focusable(element) {
  const nodeName = element.nodeName.toLowerCase();
  const isTabIndexNotNaN = !Number.isNaN(getElementTabIndex(element));
  const res = TABBABLE_NODES.test(nodeName) && !element.disabled || (element instanceof HTMLAnchorElement ? element.href || isTabIndexNotNaN : isTabIndexNotNaN);
  return res && visible(element);
}
function tabbable(element) {
  const tabIndex = getElementTabIndex(element);
  const isTabIndexNaN = Number.isNaN(tabIndex);
  return (isTabIndexNaN || tabIndex >= 0) && focusable(element);
}
function findTabbableDescendants(element) {
  return Array.from(element.querySelectorAll(FOCUS_SELECTOR)).filter(tabbable);
}

function scopeTab(node, event) {
  const tabbable = findTabbableDescendants(node);
  if (!tabbable.length) {
    event.preventDefault();
    return;
  }
  const finalTabbable = tabbable[event.shiftKey ? 0 : tabbable.length - 1];
  const leavingFinalTabbable = finalTabbable === document.activeElement || node === document.activeElement;
  if (!leavingFinalTabbable) {
    return;
  }
  event.preventDefault();
  const target = tabbable[event.shiftKey ? tabbable.length - 1 : 0];
  if (target) {
    target.focus();
  }
}

function createAriaHider(containerNode, selector = "body > :not(script)") {
  const rootNodes = Array.from(document.querySelectorAll(selector)).map((node) => {
    if (node.contains(containerNode)) {
      return void 0;
    }
    const ariaHidden = node.getAttribute("aria-hidden");
    if (ariaHidden === null || ariaHidden === "false") {
      node.setAttribute("aria-hidden", "true");
    }
    return { node, ariaHidden };
  });
  return () => {
    rootNodes.forEach((item) => {
      if (!item) {
        return;
      }
      if (item.ariaHidden === null) {
        item.node.removeAttribute("aria-hidden");
      } else {
        item.node.setAttribute("aria-hidden", item.ariaHidden);
      }
    });
  };
}

function useFocusTrap(active = true) {
  const ref = useRef();
  const restoreAria = useRef(null);
  const setRef = useCallback((node) => {
    if (!active) {
      return;
    }
    if (restoreAria.current) {
      restoreAria.current();
    }
    if (node) {
      const processNode = (_node) => {
        restoreAria.current = createAriaHider(_node);
        let focusElement = node.querySelector("[data-autofocus]");
        if (!focusElement) {
          const children = Array.from(node.querySelectorAll(FOCUS_SELECTOR));
          focusElement = children.find(tabbable) || children.find(focusable) || null;
          if (!focusElement && focusable(node))
            focusElement = node;
        }
        if (focusElement) {
          focusElement.focus();
        } else if (process.env.NODE_ENV === "development") {
          console.warn("[@mantine/hooks/use-focus-trap] Failed to find focusable element within provided node", node);
        }
      };
      setTimeout(() => {
        if (node.ownerDocument) {
          processNode(node);
        } else if (process.env.NODE_ENV === "development") {
          console.warn("[@mantine/hooks/use-focus-trap] Ref node is not part of the dom", node);
        }
      });
      ref.current = node;
    } else {
      ref.current = null;
    }
  }, [active]);
  useEffect(() => {
    if (!active) {
      return void 0;
    }
    const handleKeyDown = (event) => {
      if (event.key === "Tab" && ref.current) {
        scopeTab(ref.current, event);
      }
    };
    document.addEventListener("keydown", handleKeyDown);
    return () => {
      document.removeEventListener("keydown", handleKeyDown);
    };
  }, [active]);
  return setRef;
}

var styles = {"modal":"Modal-module__i5Pbh60m__v6-1-0","modalBackdrop":"Modal-module__ZVIwEMrU__v6-1-0","modalContent":"Modal-module__3UHa_kcB__v6-1-0","scrollLocked":"Modal-module__1T2Qjbem__v6-1-0"};

var Modal = function (_a) {
    var metadata = _a.metadata, children = _a.children, className = _a.className, onRequestToClose = _a.onRequestToClose, _b = _a.onClose, onClose = _b === void 0 ? noop : _b, _c = _a.onOpen, onOpen = _c === void 0 ? noop : _c, open = _a.open;
    var ref = useFocusTrap();
    var firstRender = useRef(true);
    useEventListener('keydown', function (event) {
        if (open && event.key === 'Escape')
            onRequestToClose();
    }, window);
    useEffect(function () {
        if (firstRender.current) {
            firstRender.current = false;
            if (!open)
                return;
        }
        if (open) {
            onOpen();
            invoke(document, 'body.classList.add', styles.scrollLocked);
        }
        else {
            onClose();
            invoke(document, 'body.classList.remove', styles.scrollLocked);
        }
        // eslint-disable-next-line consistent-return
        return function () {
            invoke(document, 'body.classList.remove', styles.scrollLocked);
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [open]);
    if (!open)
        return null;
    return (React.createElement(GroupedPortal, null,
        React.createElement("div", __assign({}, parseMetadata(metadata), { className: styles.modal, ref: ref }),
            React.createElement("div", { className: styles.modalBackdrop }),
            React.createElement("div", { className: classnames(styles.modalContent, className) }, children))));
};

var DEFAULT_LABEL_ID = 'dialog-title';
var DialogButton = function (_a) {
    var _b, _c;
    var onClick = _a.onClick, buttonState = _a.buttonState, buttonLabel = _a.buttonLabel, buttonIcon = _a.buttonIcon, buttonAriaLabel = _a.buttonAriaLabel, buttonType = _a.buttonType, buttonForm = _a.buttonForm, buttonMetadata = _a.metadata, isPrimary = _a.isPrimary, variant = _a.variant, tooltip = _a.tooltip, onDialogButtonFocus = _a.onDialogButtonFocus;
    var getTheme = function () {
        if (!isPrimary)
            return 'default';
        return variant === 'destructive' ? 'danger' : 'primary';
    };
    var button = buttonIcon ? (React.createElement(IconButton, __assign({ className: classnames(styles$1.footerAction, (_b = {},
            _b[styles$1.disabled] = buttonState === 'disabled',
            _b['dialog-danger'] = isPrimary && variant === 'destructive',
            _b['dialog-primary'] = isPrimary && variant !== 'destructive',
            _b)), type: buttonType, form: buttonForm, onClick: onClick, icon: buttonIcon, 
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        ariaLabel: buttonAriaLabel, buttonState: buttonState, theme: getTheme() }, parseMetadata(buttonMetadata), { onFocus: onDialogButtonFocus }))) : (React.createElement(Button, __assign({ className: classnames(styles$1.footerAction, (_c = {},
            _c[styles$1.disabled] = buttonState === 'disabled',
            _c['dialog-danger'] = isPrimary && variant === 'destructive',
            _c['dialog-primary'] = isPrimary && variant !== 'destructive',
            _c)), type: buttonType, form: buttonForm, onClick: onClick, buttonState: buttonState, theme: getTheme() }, parseMetadata(buttonMetadata), { onFocus: onDialogButtonFocus }), buttonLabel));
    if (tooltip) {
        return (React.createElement(Tooltip, __assign({}, tooltip),
            React.createElement("div", null, button)));
    }
    return button;
};
var Dialog = function (_a) {
    var _b, _c;
    var metadata = _a.metadata, children = _a.children, className = _a.className, onRequestToClose = _a.onRequestToClose, onClose = _a.onClose, onOpen = _a.onOpen, open = _a.open, title = _a.title, _d = _a.labelledBy, labelledBy = _d === void 0 ? DEFAULT_LABEL_ID : _d, describedBy = _a.describedBy, primaryButton = _a.primaryButton, secondaryButtons = _a.secondaryButtons, _e = _a.variant, variant = _e === void 0 ? 'default' : _e, _f = _a.size, size = _f === void 0 ? 'small' : _f;
    var _g = useState(), closeButtonElement = _g[0], setCloseButtonElement = _g[1];
    var _h = useState(false), topShadow = _h[0], setTopShadow = _h[1];
    var _j = useState(false), bottomShadow = _j[0], setBottomShadow = _j[1];
    var setElement = useScrollObserver(function (percent) {
        setTopShadow(percent > 0);
        setBottomShadow(percent < 100);
    }).setElement;
    var getDialogButtons = function () {
        var primaryButtonArray = primaryButton
            ? [__assign(__assign({}, primaryButton), { isPrimary: true })]
            : [];
        var secondaryButtonsArray = (secondaryButtons === null || secondaryButtons === void 0 ? void 0 : secondaryButtons.map(function (button) { return (__assign(__assign({}, button), { isPrimary: false })); })) || [];
        return primaryButtonArray.concat(secondaryButtonsArray.reverse());
    };
    return (React.createElement(Modal, { metadata: metadata, className: className, onClose: onClose, onOpen: onOpen, open: open, onRequestToClose: onRequestToClose },
        React.createElement("section", { role: "dialog", "aria-modal": "true", "aria-labelledby": labelledBy, "aria-describedby": describedBy, className: classnames(styles$1.dialog, styles$1["size-" + size]) },
            React.createElement("header", { className: classnames(styles$1.header, (_b = {},
                    _b[styles$1.headerShadowed] = topShadow,
                    _b)) },
                React.createElement(Typography, { component: "span", variant: "heading-3", className: styles$1.headerTitle, id: labelledBy }, title),
                React.createElement(IconButton, __assign({ ref: setCloseButtonElement, icon: "times", ariaLabel: "Close Dialog", className: styles$1.closeButton, onClick: onRequestToClose, tabIndex: -1 }, parseMetadata(suffixMetadata(metadata, 'close-button'))))),
            React.createElement("div", { ref: setElement, className: styles$1.body }, children),
            React.createElement("footer", { className: classnames(styles$1.footer, (_c = {},
                    _c[styles$1.footerShadowed] = bottomShadow,
                    _c)) }, getDialogButtons().map(function (button, index) { return (React.createElement(DialogButton
            // eslint-disable-next-line react/no-array-index-key
            , __assign({ 
                // eslint-disable-next-line react/no-array-index-key
                key: (button.buttonLabel || button.buttonIcon) + "-" + index, variant: variant, onDialogButtonFocus: function () {
                    if (closeButtonElement)
                        closeButtonElement.tabIndex = 0;
                } }, button))); })))));
};

export { Dialog, Modal };
//# sourceMappingURL=index.js.map
